@@ -5,7 +5,7 @@ def get_td_Home():
     # Returns the path where this file is stored
     return os.path.realpath(__file__).strip("data_utils.py")
 
-def read_rna_file(rna_file_path, identifier='hugo', fetch_missing_hugo=True):
+def read_rna_file(rna_file_path, identifier='hugo', fetch_missing_hugo=False):
     """
     Read in a cbioportal pancancer or TCGA Xena Hug txt file containing mixture gene expression data, and return a pandas dataframe.
     Columns are patients, Rows are genes
@@ -40,7 +40,7 @@ def read_rna_file(rna_file_path, identifier='hugo', fetch_missing_hugo=True):
                 rna = hugo(rna)
             rna = rna.set_index(['Hugo_Symbol']).drop(['Entrez_Gene_Id'], axis=1)
             # Remove rows/genes that don't have a Hugo Symbol name:
-            rna = rna.drop('nan')
+            rna = rna.drop('nan',errors='ignore') # ignore error that arrises if no 'nan's
         elif identifier == 'entrez':
             rna = rna.set_index(['Entrez_Gene_Id']).drop(['Hugo_Symbol'], axis=1)
             # Remove rows/genes that don't have a Entrez ID:
@@ -168,28 +168,128 @@ def download_from_xena(url="https://tcga.xenahubs.net/download/TCGA.UCS.sampleMa
     return df
 
 
-def read_lm22_file(file_path=get_td_Home()+"data/LM22.txt"):
+# # OBSOLETE! Use read_sig_file() with no arguments to read in LM22.txt instead
+# def read_lm22_file(file_path=get_td_Home()+"data/LM22.txt"):
+#     """
+#     Read in the LM22 (or LM6) signature matrix file (containing signature gene expression data for 22 different cell types)
+#     and return a pandas dataframe
+#     Inputs:
+#         - file_path: string. Relative or full path to the LM22/LM6 signature matrix file
+#             This file is tab seperated. Columns are cell types, Rows are gene Hugo Symbols.
+#     Output:
+#         - pandas data frame. Columns are cell types, Rows are genes, indexed by Hugo Symbol.
+#     """
+#
+#     import pandas as pd
+#
+#     print("WARNING: 'read_lm22_file()'' depreciated. Use 'read_sig_file()' with no arguments to read in 'LM22.txt' signatures")
+#
+#     lm22 = pd.read_csv(file_path, sep='\t')
+#     # lm22['Hugo_Symbol'] = lm22['Gene symbol']
+#     # lm22 = lm22.drop(['Gene symbol'], axis = 1)
+#     new_column_names = lm22.columns.tolist()
+#     new_column_names[0] = 'Hugo_Symbol'
+#     lm22.columns = new_column_names
+#     lm22 = lm22.set_index(['Hugo_Symbol'])
+#
+#     return lm22
+
+
+def read_sig_file(file_path=get_td_Home()+"data/LM22.txt", geneID="Hugo_Symbol"):
     """
-    Read in the LM22 (or LM6) signature matrix file (containing signature gene expression data for 22 different cell types)
-    and return a pandas dataframe
+    Reads in a signature matrix file (containing signature gene expression data for
+    a number of different cell types), converts the gene identifiers to Hugo Symbols,
+    and returns a pandas dataframe.
+        ** IF NO FILE PATH GIVEN, LM22 IS ASSUMED **
     Inputs:
-        - file_path: string. Relative or full path to the LM22/LM6 signature matrix file
-            This file is tab seperated. Columns are cell types, Rows are gene Hugo Symbols.
+        - file_path: string. Relative or full path to signature matrix file.
+            This file is tab seperated. Columns are cell types, Rows are genes
+            If no file_path given, LM22 is assumed.
+        - geneID: string in ["Hugo_Symbol", "Ensembl_Gene_ID", "Entrez_Gene_ID"].
+            Describes how genes are labeled in the signature matrix file
     Output:
         - pandas data frame. Columns are cell types, Rows are genes, indexed by Hugo Symbol.
     """
-
     import pandas as pd
 
-    lm22 = pd.read_csv(file_path, sep='\t')
-    # lm22['Hugo_Symbol'] = lm22['Gene symbol']
-    # lm22 = lm22.drop(['Gene symbol'], axis = 1)
-    new_column_names = lm22.columns.tolist()
-    new_column_names[0] = 'Hugo_Symbol'
-    lm22.columns = new_column_names
-    lm22 = lm22.set_index(['Hugo_Symbol'])
+    # Read file:
+    sig = pd.read_csv(file_path, sep='\t')
+    # Get name of column with gene identifiers:
+    gene_col = list(sig)[0]
+
+    to_be_dropped = []
+    if geneID == "Hugo_Symbol":
+        sig['Hugo_Symbol'] = sig[gene_col]
+
+    elif geneID == 'Ensembl_Gene_ID':
+        sig['Hugo_Symbol'], to_be_dropped = convert_emsembl(sig[gene_col])
+
+    elif geneID == 'Entrez_Gene_ID':
+        raise ValueError("Conversion from Entrez Gene ID to Hugo Symbol not yet supported in TumorDecon")
+
+    else:
+        raise ValueError("geneID ({!r}) must be set to 'Hugo_Symbol', 'Ensembl_Gene_ID', or 'Entrez_Gene_ID'".format(geneID))
+
+    # Drop old gene ID column, rename columns, and set Index to Hugo Symbol:
+    sig = sig.drop([gene_col], axis = 1)
+    new_column_names = sig.columns.tolist()
+    sig.columns = new_column_names
+    sig = sig.set_index(['Hugo_Symbol'])
+
+    # Drop columns without a Hugo Symbol:
+    sig = sig.drop(to_be_dropped)
+    return sig
+
+def convert_emsembl(ensembl):
+    """
+    Input:
+        - ordered array of Ensembl Gene IDs to be converted to Hugo Symbols
+    Outputs:
+        - hugo: array. Ordered array of Hugo Symbols correponsing to the Ensembl Gene IDs
+        - to_be_dropped: array. List of genes without a correponsing hugo symbol (to be dropped)
+    """
+    import pandas as pd
+    import numpy as np
+
+    hugo = np.array(ensembl)
+    # Will drop any genes that don't have a corresponding Hugo Symbol
+    to_be_dropped = []
+
+    # Create dictionary for conversion:
+    ensembl_hugo = pd.read_csv(get_td_Home()+"data/gene_ID_conversion/hugo_enseml_synonym.txt", header=0, sep="\t")
+    ensembl_hugo2 = pd.read_csv(get_td_Home()+"data/gene_ID_conversion/GSE87692_Primary_HsESF_TPMs_073015.txt", header=0, sep="\t")
+    ensembl_hugo2 = ensembl_hugo2[['ensemble_geneid', 'Associate Gene Name']]
+    ensembl_hugo2.set_index('Associate Gene Name', inplace=True)
+    ensembl_hugo.dropna(subset=['Ensembl ID(supplied by Ensembl)', 'Ensembl gene ID'], how='all', inplace=True)
+    ensembl_hugo = ensembl_hugo.set_index('Approved symbol')
+
+    for i, gene in enumerate(ensembl):
+        # first search in the ensembl_hugo2 then if not found in ensembl_hugo
+        is_found = ensembl_hugo2.loc[ensembl_hugo2['ensemble_geneid'] == gene]
+        if len(is_found) > 0 and is_found.index[0] != "#N/A":
+            hugo[i] = is_found.index[0]
+
+        else:
+            t = ensembl_hugo.loc[ensembl_hugo['Ensembl ID(supplied by Ensembl)'] == gene]
+            # first search in 'Ensembl ID(supplied by Ensembl)' column, if not found, search the other col
+            if len(t) > 0:
+                if type(t.index[0]) == str:
+                    hugo[i] = t.index[0]
+
+            else:
+                t2 = ensembl_hugo.loc[ensembl_hugo['Ensembl gene ID'] == gene]
+                if len(t2) > 0:
+                    if type(t2.index[0]) == str:
+                        hugo[i] = t2.index[0]
+                    else:
+                        to_be_dropped.append(gene)
+                else:
+                    to_be_dropped.append(gene)
+    if len(to_be_dropped) > 0:
+        print("Missing Hugo Symbols for the following genes (will be dropped from signature matrix):")
+        print(to_be_dropped)
+    return hugo, to_be_dropped
 
-    return lm22
 
 def df_normalization(df, scaling, axis=0):
     """
@@ -382,6 +482,51 @@ def read_geneset(filepath):
     return chosen_genes
 
 
+def combine_celltypes(df, cols_to_combine=None):
+    """
+    Function to sum related cell types into a single column
+    Inputs:
+        - df: pandas dataframe. Output of td.tumor_deconvolve()
+        - cols_to_combine: dictionary. Keys are the desired names of any new cell type columns,
+            values are an arary of current column names to combine under the key name
+                - Default = dictionary for combining common cell types from LM22
+    Outputs:
+        - Pandas dataframe with columns combined as specified by the dictionary.
+            All unmentioned column names are left as they are
+    """
+    import pandas as pd
+
+    if cols_to_combine is not None:
+        from collections.abc import Mapping
+        if isinstance(cols_to_combine, Mapping):
+            pass
+        else:
+            raise TypeError("cols_to_combine must be a dictionary")
+    else:
+        # Use LM22 as default
+        print("WARNING: No dictionary defined for combining columns... Attempting to use default dict for LM22 signatures")
+        cols_to_combine = {'B cells':['B cells naive', 'B cells memory'],
+                           'CD4 T cells':['T cells CD4 naive', 'T cells CD4 memory resting', 'T cells CD4 memory activated','T cells follicular helper', 'T cells regulatory (Tregs)'],
+                           'CD8 T cells':['T cells CD8'],
+                           'NK cells':['NK cells resting', 'NK cells activated'],
+                           'Macrophages':['Macrophages M0', 'Macrophages M1', 'Macrophages M2'],
+                           'Mast cells':['Mast cells resting','Mast cells activated'],
+                           'Dendritic cells':['Dendritic cells resting', 'Dendritic cells activated']
+                          }
+    df2 = df.copy()
+    for cell_type in cols_to_combine.keys():
+        try:
+            df2[cell_type] = df2[cols_to_combine[cell_type]].sum(axis=1)
+            df2.drop(cols_to_combine[cell_type],axis=1,inplace=True)
+        except KeyError as e:
+            print("WARNING: Failed to combine some columns: ")
+            print("KeyError: "+str(e))
+            pass
+
+    return df2
+
+
+
 def corr_table(methods, results, cell_types, true_freqs):
     import pandas as pd
     import numpy as np
